# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import util_intstr
from . import meta_v1
from . import core_v1
from . import runtime


@dataclass
class ControllerRevision(DictMixin):
    r"""DEPRECATED - This group version of ControllerRevision is deprecated by
      apps/v1beta2/ControllerRevision. See the release notes for more information.
      ControllerRevision implements an immutable snapshot of state data. Clients are
      responsible for serializing and deserializing the objects that contain their
      internal state. Once a ControllerRevision has been successfully created, it
      can not be updated. The API Server will fail validation of all requests that
      attempt to mutate the Data field. ControllerRevisions may, however, be
      deleted. Note that, due to its use by both the DaemonSet and StatefulSet
      controllers for update and rollback, this object is beta. However, it may be
      subject to name and representation changes in future releases, and clients
      should not depend on its stability. It is primarily for internal use by
      controllers.

      **parameters**

      * **revision** ``int`` - Revision indicates the revision of the state represented by Data.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **data** ``Optional[runtime.RawExtension]`` - Data is the serialized representation of the state.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata
    """
    revision: 'int'
    apiVersion: 'Optional[str]' = None
    data: 'Optional[runtime.RawExtension]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class ControllerRevisionList(DictMixin):
    r"""ControllerRevisionList is a resource containing a list of ControllerRevision
      objects.

      **parameters**

      * **items** ``List[ControllerRevision]`` - Items is the list of ControllerRevisions
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata
    """
    items: 'List[ControllerRevision]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class Deployment(DictMixin):
    r"""DEPRECATED - This group version of Deployment is deprecated by
      apps/v1beta2/Deployment. See the release notes for more information.
      Deployment enables declarative updates for Pods and ReplicaSets.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata.
      * **spec** ``Optional[DeploymentSpec]`` - Specification of the desired behavior of the Deployment.
      * **status** ``Optional[DeploymentStatus]`` - Most recently observed status of the Deployment.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[DeploymentSpec]' = None
    status: 'Optional[DeploymentStatus]' = None


@dataclass
class DeploymentCondition(DictMixin):
    r"""DeploymentCondition describes the state of a deployment at a certain point.

      **parameters**

      * **status** ``str`` - Status of the condition, one of True, False, Unknown.
      * **type** ``str`` - Type of deployment condition.
      * **lastTransitionTime** ``Optional[meta_v1.Time]`` - Last time the condition transitioned from one status to another.
      * **lastUpdateTime** ``Optional[meta_v1.Time]`` - The last time this condition was updated.
      * **message** ``Optional[str]`` - A human readable message indicating details about the transition.
      * **reason** ``Optional[str]`` - The reason for the condition's last transition.
    """
    status: 'str'
    type: 'str'
    lastTransitionTime: 'Optional[meta_v1.Time]' = None
    lastUpdateTime: 'Optional[meta_v1.Time]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class DeploymentList(DictMixin):
    r"""DeploymentList is a list of Deployments.

      **parameters**

      * **items** ``List[Deployment]`` - Items is the list of Deployments.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata.
    """
    items: 'List[Deployment]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class DeploymentRollback(DictMixin):
    r"""DEPRECATED. DeploymentRollback stores the information required to rollback a
      deployment.

      **parameters**

      * **name** ``str`` - Required: This must match the Name of a deployment.
      * **rollbackTo** ``RollbackConfig`` - The config of this deployment rollback.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **updatedAnnotations** ``Optional[dict]`` - The annotations to be updated to a deployment
    """
    name: 'str'
    rollbackTo: 'RollbackConfig'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    updatedAnnotations: 'Optional[dict]' = None


@dataclass
class DeploymentSpec(DictMixin):
    r"""DeploymentSpec is the specification of the desired behavior of the Deployment.

      **parameters**

      * **template** ``core_v1.PodTemplateSpec`` - Template describes the pods that will be created.
      * **minReadySeconds** ``Optional[int]`` - Minimum number of seconds for which a newly created pod should be ready
        without any of its container crashing, for it to be considered available.
        Defaults to 0 (pod will be considered available as soon as it is ready)
      * **paused** ``Optional[bool]`` - Indicates that the deployment is paused.
      * **progressDeadlineSeconds** ``Optional[int]`` - The maximum time in seconds for a deployment to make progress before it is
        considered to be failed. The deployment controller will continue to process
        failed deployments and a condition with a ProgressDeadlineExceeded reason will
        be surfaced in the deployment status. Note that progress will not be estimated
        during the time a deployment is paused. Defaults to 600s.
      * **replicas** ``Optional[int]`` - Number of desired pods. This is a pointer to distinguish between explicit zero
        and not specified. Defaults to 1.
      * **revisionHistoryLimit** ``Optional[int]`` - The number of old ReplicaSets to retain to allow rollback. This is a pointer
        to distinguish between explicit zero and not specified. Defaults to 2.
      * **rollbackTo** ``Optional[RollbackConfig]`` - DEPRECATED. The config this deployment is rolling back to. Will be cleared
        after rollback is done.
      * **selector** ``Optional[meta_v1.LabelSelector]`` - Label selector for pods. Existing ReplicaSets whose pods are selected by this
        will be the ones affected by this deployment.
      * **strategy** ``Optional[DeploymentStrategy]`` - The deployment strategy to use to replace existing pods with new ones.
    """
    template: 'core_v1.PodTemplateSpec'
    minReadySeconds: 'Optional[int]' = None
    paused: 'Optional[bool]' = None
    progressDeadlineSeconds: 'Optional[int]' = None
    replicas: 'Optional[int]' = None
    revisionHistoryLimit: 'Optional[int]' = None
    rollbackTo: 'Optional[RollbackConfig]' = None
    selector: 'Optional[meta_v1.LabelSelector]' = None
    strategy: 'Optional[DeploymentStrategy]' = None


@dataclass
class DeploymentStatus(DictMixin):
    r"""DeploymentStatus is the most recently observed status of the Deployment.

      **parameters**

      * **availableReplicas** ``Optional[int]`` - Total number of available pods (ready for at least minReadySeconds) targeted
        by this deployment.
      * **collisionCount** ``Optional[int]`` - Count of hash collisions for the Deployment. The Deployment controller uses
        this field as a collision avoidance mechanism when it needs to create the name
        for the newest ReplicaSet.
      * **conditions** ``Optional[List[DeploymentCondition]]`` - Represents the latest available observations of a deployment's current state.
      * **observedGeneration** ``Optional[int]`` - The generation observed by the deployment controller.
      * **readyReplicas** ``Optional[int]`` - Total number of ready pods targeted by this deployment.
      * **replicas** ``Optional[int]`` - Total number of non-terminated pods targeted by this deployment (their labels
        match the selector).
      * **unavailableReplicas** ``Optional[int]`` - Total number of unavailable pods targeted by this deployment. This is the
        total number of pods that are still required for the deployment to have 100%
        available capacity. They may either be pods that are running but not yet
        available or pods that still have not been created.
      * **updatedReplicas** ``Optional[int]`` - Total number of non-terminated pods targeted by this deployment that have the
        desired template spec.
    """
    availableReplicas: 'Optional[int]' = None
    collisionCount: 'Optional[int]' = None
    conditions: 'Optional[List[DeploymentCondition]]' = None
    observedGeneration: 'Optional[int]' = None
    readyReplicas: 'Optional[int]' = None
    replicas: 'Optional[int]' = None
    unavailableReplicas: 'Optional[int]' = None
    updatedReplicas: 'Optional[int]' = None


@dataclass
class DeploymentStrategy(DictMixin):
    r"""DeploymentStrategy describes how to replace existing pods with new ones.

      **parameters**

      * **rollingUpdate** ``Optional[RollingUpdateDeployment]`` - Rolling update config params. Present only if DeploymentStrategyType =
        RollingUpdate.
      * **type** ``Optional[str]`` - Type of deployment. Can be "Recreate" or "RollingUpdate". Default is
        RollingUpdate.
    """
    rollingUpdate: 'Optional[RollingUpdateDeployment]' = None
    type: 'Optional[str]' = None


@dataclass
class RollbackConfig(DictMixin):
    r"""DEPRECATED.

      **parameters**

      * **revision** ``Optional[int]`` - The revision to rollback to. If set to 0, rollback to the last revision.
    """
    revision: 'Optional[int]' = None


@dataclass
class RollingUpdateDeployment(DictMixin):
    r"""Spec to control the desired behavior of rolling update.

      **parameters**

      * **maxSurge** ``Optional[util_intstr.IntOrString]`` - The maximum number of pods that can be scheduled above the desired number of
        pods. Value can be an absolute number (ex: 5) or a percentage of desired pods
        (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is
        calculated from percentage by rounding up. Defaults to 25%. Example: when this
        is set to 30%, the new ReplicaSet can be scaled up immediately when the
        rolling update starts, such that the total number of old and new pods do not
        exceed 130% of desired pods. Once old pods have been killed, new ReplicaSet
        can be scaled up further, ensuring that total number of pods running at any
        time during the update is at most 130% of desired pods.
      * **maxUnavailable** ``Optional[util_intstr.IntOrString]`` - The maximum number of pods that can be unavailable during the update. Value
        can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%).
        Absolute number is calculated from percentage by rounding down. This can not
        be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the
        old ReplicaSet can be scaled down to 70% of desired pods immediately when the
        rolling update starts. Once new pods are ready, old ReplicaSet can be scaled
        down further, followed by scaling up the new ReplicaSet, ensuring that the
        total number of pods available at all times during the update is at least 70%
        of desired pods.
    """
    maxSurge: 'Optional[util_intstr.IntOrString]' = None
    maxUnavailable: 'Optional[util_intstr.IntOrString]' = None


@dataclass
class RollingUpdateStatefulSetStrategy(DictMixin):
    r"""RollingUpdateStatefulSetStrategy is used to communicate parameter for
      RollingUpdateStatefulSetStrategyType.

      **parameters**

      * **partition** ``Optional[int]`` - Partition indicates the ordinal at which the StatefulSet should be
        partitioned.
    """
    partition: 'Optional[int]' = None


@dataclass
class Scale(DictMixin):
    r"""Scale represents a scaling request for a resource.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#metadata.
      * **spec** ``Optional[ScaleSpec]`` - defines the behavior of the scale. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.
      * **status** ``Optional[ScaleStatus]`` - current status of the scale. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#spec-and-status.
        Read-only.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[ScaleSpec]' = None
    status: 'Optional[ScaleStatus]' = None


@dataclass
class ScaleSpec(DictMixin):
    r"""ScaleSpec describes the attributes of a scale subresource

      **parameters**

      * **replicas** ``Optional[int]`` - desired number of instances for the scaled object.
    """
    replicas: 'Optional[int]' = None


@dataclass
class ScaleStatus(DictMixin):
    r"""ScaleStatus represents the current status of a scale subresource.

      **parameters**

      * **replicas** ``int`` - actual number of observed instances of the scaled object.
      * **selector** ``Optional[dict]`` - label query over pods that should match the replicas count. More info:
        http://kubernetes.io/docs/user-guide/labels#label-selectors
      * **targetSelector** ``Optional[str]`` - label selector for pods that should match the replicas count. This is a
        serializated version of both map-based and more expressive set-based
        selectors. This is done to avoid introspection in the clients. The string will
        be in the same format as the query-param syntax. If the target type only
        supports map-based selectors, both this field and map-based selector field are
        populated. More info:
        https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors
    """
    replicas: 'int'
    selector: 'Optional[dict]' = None
    targetSelector: 'Optional[str]' = None


@dataclass
class StatefulSet(DictMixin):
    r"""DEPRECATED - This group version of StatefulSet is deprecated by
      apps/v1beta2/StatefulSet. See the release notes for more information.
      StatefulSet represents a set of pods with consistent identities. Identities
      are defined as:
       - Network: A single stable DNS and hostname.
       - Storage: As many VolumeClaims as requested.
      The StatefulSet guarantees that a given network identity will always map to
      the same storage identity.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - 
      * **spec** ``Optional[StatefulSetSpec]`` - Spec defines the desired identities of pods in this set.
      * **status** ``Optional[StatefulSetStatus]`` - Status is the current status of Pods in this StatefulSet. This data may be out
        of date by some window of time.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[StatefulSetSpec]' = None
    status: 'Optional[StatefulSetStatus]' = None


@dataclass
class StatefulSetCondition(DictMixin):
    r"""StatefulSetCondition describes the state of a statefulset at a certain point.

      **parameters**

      * **status** ``str`` - Status of the condition, one of True, False, Unknown.
      * **type** ``str`` - Type of statefulset condition.
      * **lastTransitionTime** ``Optional[meta_v1.Time]`` - Last time the condition transitioned from one status to another.
      * **message** ``Optional[str]`` - A human readable message indicating details about the transition.
      * **reason** ``Optional[str]`` - The reason for the condition's last transition.
    """
    status: 'str'
    type: 'str'
    lastTransitionTime: 'Optional[meta_v1.Time]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class StatefulSetList(DictMixin):
    r"""StatefulSetList is a collection of StatefulSets.

      **parameters**

      * **items** ``List[StatefulSet]`` - 
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - 
    """
    items: 'List[StatefulSet]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class StatefulSetSpec(DictMixin):
    r"""A StatefulSetSpec is the specification of a StatefulSet.

      **parameters**

      * **serviceName** ``str`` - serviceName is the name of the service that governs this StatefulSet. This
        service must exist before the StatefulSet, and is responsible for the network
        identity of the set. Pods get DNS/hostnames that follow the pattern:
        pod-specific-string.serviceName.default.svc.cluster.local where
        "pod-specific-string" is managed by the StatefulSet controller.
      * **template** ``core_v1.PodTemplateSpec`` - template is the object that describes the pod that will be created if
        insufficient replicas are detected. Each pod stamped out by the StatefulSet
        will fulfill this Template, but have a unique identity from the rest of the
        StatefulSet.
      * **podManagementPolicy** ``Optional[str]`` - podManagementPolicy controls how pods are created during initial scale up,
        when replacing pods on nodes, or when scaling down. The default policy is
        `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1,
        etc) and the controller will wait until each pod is ready before continuing.
        When scaling down, the pods are removed in the opposite order. The alternative
        policy is `Parallel` which will create pods in parallel to match the desired
        scale without waiting, and on scale down will delete all pods at once.
      * **replicas** ``Optional[int]`` - replicas is the desired number of replicas of the given Template. These are
        replicas in the sense that they are instantiations of the same Template, but
        individual replicas also have a consistent identity. If unspecified, defaults
        to 1.
      * **revisionHistoryLimit** ``Optional[int]`` - revisionHistoryLimit is the maximum number of revisions that will be
        maintained in the StatefulSet's revision history. The revision history
        consists of all revisions not represented by a currently applied
        StatefulSetSpec version. The default value is 10.
      * **selector** ``Optional[meta_v1.LabelSelector]`` - selector is a label query over pods that should match the replica count. If
        empty, defaulted to labels on the pod template. More info:
        https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors
      * **updateStrategy** ``Optional[StatefulSetUpdateStrategy]`` - updateStrategy indicates the StatefulSetUpdateStrategy that will be employed
        to update Pods in the StatefulSet when a revision is made to Template.
      * **volumeClaimTemplates** ``Optional[List[core_v1.PersistentVolumeClaim]]`` - volumeClaimTemplates is a list of claims that pods are allowed to reference.
        The StatefulSet controller is responsible for mapping network identities to
        claims in a way that maintains the identity of a pod. Every claim in this list
        must have at least one matching (by name) volumeMount in one container in the
        template. A claim in this list takes precedence over any volumes in the
        template, with the same name.
    """
    serviceName: 'str'
    template: 'core_v1.PodTemplateSpec'
    podManagementPolicy: 'Optional[str]' = None
    replicas: 'Optional[int]' = None
    revisionHistoryLimit: 'Optional[int]' = None
    selector: 'Optional[meta_v1.LabelSelector]' = None
    updateStrategy: 'Optional[StatefulSetUpdateStrategy]' = None
    volumeClaimTemplates: 'Optional[List[core_v1.PersistentVolumeClaim]]' = None


@dataclass
class StatefulSetStatus(DictMixin):
    r"""StatefulSetStatus represents the current state of a StatefulSet.

      **parameters**

      * **replicas** ``int`` - replicas is the number of Pods created by the StatefulSet controller.
      * **collisionCount** ``Optional[int]`` - collisionCount is the count of hash collisions for the StatefulSet. The
        StatefulSet controller uses this field as a collision avoidance mechanism when
        it needs to create the name for the newest ControllerRevision.
      * **conditions** ``Optional[List[StatefulSetCondition]]`` - Represents the latest available observations of a statefulset's current state.
      * **currentReplicas** ``Optional[int]`` - currentReplicas is the number of Pods created by the StatefulSet controller
        from the StatefulSet version indicated by currentRevision.
      * **currentRevision** ``Optional[str]`` - currentRevision, if not empty, indicates the version of the StatefulSet used
        to generate Pods in the sequence [0,currentReplicas).
      * **observedGeneration** ``Optional[int]`` - observedGeneration is the most recent generation observed for this
        StatefulSet. It corresponds to the StatefulSet's generation, which is updated
        on mutation by the API Server.
      * **readyReplicas** ``Optional[int]`` - readyReplicas is the number of Pods created by the StatefulSet controller that
        have a Ready Condition.
      * **updateRevision** ``Optional[str]`` - updateRevision, if not empty, indicates the version of the StatefulSet used to
        generate Pods in the sequence [replicas-updatedReplicas,replicas)
      * **updatedReplicas** ``Optional[int]`` - updatedReplicas is the number of Pods created by the StatefulSet controller
        from the StatefulSet version indicated by updateRevision.
    """
    replicas: 'int'
    collisionCount: 'Optional[int]' = None
    conditions: 'Optional[List[StatefulSetCondition]]' = None
    currentReplicas: 'Optional[int]' = None
    currentRevision: 'Optional[str]' = None
    observedGeneration: 'Optional[int]' = None
    readyReplicas: 'Optional[int]' = None
    updateRevision: 'Optional[str]' = None
    updatedReplicas: 'Optional[int]' = None


@dataclass
class StatefulSetUpdateStrategy(DictMixin):
    r"""StatefulSetUpdateStrategy indicates the strategy that the StatefulSet
      controller will use to perform updates. It includes any additional parameters
      necessary to perform the update for the indicated strategy.

      **parameters**

      * **rollingUpdate** ``Optional[RollingUpdateStatefulSetStrategy]`` - RollingUpdate is used to communicate parameters when Type is
        RollingUpdateStatefulSetStrategyType.
      * **type** ``Optional[str]`` - Type indicates the type of the StatefulSetUpdateStrategy.
    """
    rollingUpdate: 'Optional[RollingUpdateStatefulSetStrategy]' = None
    type: 'Optional[str]' = None


